/***************************************************************************
 *                                                                         *
 *  MAXIMUS-CBCS Source Code, Version 1.02                                 *
 *  Copyright 1989, 1990 by Scott J. Dudley.  All rights reserved.         *
 *                                                                         *
 *  BORED, the line-oriented editor                                        *
 *                                                                         *
 *  For complete details of the licensing restrictions, please refer to    *
 *  the licence agreement, which is published in its entirety in MAX.C     *
 *  and LICENCE.MAX.                                                       *
 *                                                                         *
 *  USE OF THIS FILE IS SUBJECT TO THE RESTRICTIONS CONTAINED IN THE       *
 *  MAXIMUS-CBCS LICENSING AGREEMENT.  IF YOU DO NOT FIND THE TEXT OF THIS *
 *  AGREEMENT IN ANY OF THE  AFOREMENTIONED FILES, OR IF YOU DO NOT HAVE   *
 *  THESE FILES, YOU SHOULD IMMEDIATELY CONTACT THE AUTHOR AT ONE OF THE   *
 *  ADDRESSES LISTED BELOW.  IN NO EVENT SHOULD YOU PROCEED TO USE THIS    *
 *  FILE WITHOUT HAVING ACCEPTED THE TERMS OF THE MAXIMUS-CBCS LICENSING   *
 *  AGREEMENT, OR SUCH OTHER AGREEMENT AS YOU ARE ABLE TO REACH WITH THE   *
 *  AUTHOR.                                                                *
 *                                                                         *
 *  You can contact the author at one of the address listed below:         *
 *                                                                         *
 *  Scott Dudley           FidoNet  1:249/106                              *
 *  777 Downing St.        IMEXnet  89:483/202                             *
 *  Kingston, Ont.         Internet f106.n249.z1.fidonet.org               *
 *  Canada - K7M 5N3       BBS      (613) 389-8315 - HST/14.4K, 24hrs      *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <ctype.h>
#include <setjmp.h>
#include <string.h>
#include <mem.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "prog.h"
#include "alc.h"
#include "max.h"
#include "max_msg.h"


int linenum;

int pascal Bored(struct _msg *msg,int msgfile)
{
  linenum=1;
  num_lines=0;
  usrwidth=min(usr.width,LINELEN);

  max_lines=(int)min((long)MAX_LINES,(coreleft()-BORED_SAVE_MEM)/(LINELEN*2L));

  if (max_lines <= 0)   /* Something's really wrong here! */
  {
    logit(mem_nmsgb);
    Puts(mem_nmsgb);

    Press_ENTER();
    return ABORT;
  }
  else                /* Things look A-OK! */
  {
    if (setjmp(jumpto)==0) /* Really ugly, but the best way to handle errs */
    {
      if (usr.help==NOVICE)
        Display_File(0,NULL,PRM(hlp_editor));

      if (msgfile != -1)
        Load_Message(msgfile);
      else Bored_Continue();

      return (Bored_Menu(msg));
    }
    else
    {
      Free_All();
      return ABORT;
    }
  }
}


int pascal Bored_Menu(struct _msg *msg)
{
  return Display_Options(*PRM(edit_menu) ? PRM(edit_menu) : "EDIT",msg);
}



void pascal Bored_Continue(void)
{
  linenum=num_lines+1;

  Putc('\n');

  Bored_GetLine(0);     /* Get as many lines as required */
}



void pascal Bored_GetLine(int max)
{
  char string[LINELEN+1],
       nextstring[MAX_WRAPLEN+2],
       temp[LINELEN];

  int linecount,
      wrapped;

  *nextstring='\0';
  *string='\0';

  linecount=0;

  Clear_KBuffer();

  Printf(LGREEN "   \x19%c\n",usrwidth-7);

  while (max==0 || linecount < max)
  {
    if (num_lines==max_lines-2)
      Puts(s(last_line));

    if (num_lines >= max_lines-1)
    {
      Puts(toomanylines);
      return;
    }

    strcpy(string,nextstring);

    sprintf(temp,GRAY "%2d: " YELLOW,linenum);

    wrapped=FALSE;
    
    if (Input(string,INPUT_nlb_line | INPUT_nohotflash | INPUT_wordwrap,0,
          min(usr.width-3,LINELEN)-4,temp)==1)
    {
      wrapped=TRUE;
      strcpy(nextstring,string+strlen(string)+1);
    }
    else *nextstring='\0';

    if (! *string)
      break;
    else
    {
      if (Allocate_Line(linenum))
        EdMemOvfl();

      if (wrapped)
        screen[linenum][0]=SOFT_CR;

      strcpy(screen[linenum]+1,string);
      mknl(string);

      linecount++;
      linenum++;
    }
  }
}


void pascal Bored_List(struct _msg *msg)
{
  int x;
  char nonstop;

  display_line=display_col=1;
  nonstop=FALSE;

  Puts("\n\n" CLS);

  Printf(s(list_from),msg->from);
  Printf(s(list_to),msg->to);
  Printf(s(list_subj),msg->subj);

  for (x=1;x <= num_lines;x++)
  {
    if (MoreYnBreak(&nonstop,CYAN))
      break;

    Printf(s(blfmt1),x,&screen[x][1]);
    Putc('\n');
  }

  Putc('\n');
}



void pascal Bored_To(struct _msg *msg)
{
  Printf(s(bed_to),msg->to);

  if (!(prm.flags & FLAG_no_ulist) && !(area.attrib[class] & SYSMAIL))
    Puts(s(bed_lu));

  while (1)
  {
    Puts(CYAN);
    OpusGetsL(msg->to,35,s(bto));

    if (eqstr(msg->to,qmark) && !(prm.flags & FLAG_no_ulist))
      UserList();
    else if (*(msg->to))
    {
      if (!(area.attrib[class] & SYSMAIL) && !(area.attrib[class] & ECHOMAIL) &&
          (msg->attr & MSGPRIVATE) && !IsInUserList(msg->to))
        Printf(WHITE "%s\n",userdoesntexist);
      else break;
    }
  }

  Clear_KBuffer();
}


void pascal Bored_From(struct _msg *msg)
{
  Printf(s(bfrom),msg->from);

  while (1)
  {
    Puts(CYAN);
    OpusGetsL(msg->from,35,s(bfromsp));

    if (*(msg->from))
      break;
  }

  Clear_KBuffer();
}


void pascal Bored_Subject(struct _msg *msg)
{
  Printf(s(bsubj),msg->subj);

  while (1)
  {
    Puts(CYAN);
    OpusGetsL(msg->subj,35,s(bsubject));

    if (*(msg->subj))
      break;
  }

  Clear_KBuffer();
}

void pascal Bored_Handling(struct _msg *msg)
{
  int x,y,
      ch;

  while (1)
  {

    if (! *linebuf)
    {
      Puts("\n" CLS);

      for (x=0;x < 16;x++)
      {
        Printf(YELLOW "%c" GRAY ")%s",msg_attr[x][0],&msg_attr[x][1]);

        for (y=0;y < 20-(strlen(msg_attr[x])+1);y++)
          Putc('.');

        Printf(WHITE " %s\n",(msg->attr & (1 << x)) ? yep : nope);
      }

      Puts(s(edlist_quit));
    }

    ch=KeyGetRNP(select_p);

    if ((ch=toupper(ch))=='Q' || ch=='\0' || ch=='|' || ch=='\r')
      return;
    else
    {
      for (x=0;x < 16;x++)
      {
        if (ch==msg_attr[x][0])
        {
          if (msg->attr & (1 << x))
            msg->attr &= ~(1 << x);
          else msg->attr |= (1 << x);

          break;
        }
      }

      if (x==16)
      {
        Printf(dontunderstand,ch);
        Press_ENTER();
      }
    }
  }
}


void pascal Bored_Delete(void)
{
  char temp[BUFLEN];
  int cx,x;

  WhiteN();

  OpusGets(temp,s(line_to_del));

  cx=atoi(temp);

  if (cx > 0 && cx <= num_lines)
  {
    if (! *linebuf)
    {
      Putc('\n');
      Printf(s(blfmt1),cx,screen[cx]+1);
      Puts("\n\n" WHITE);
    }

    if (GetYnAnswer(s(del_this),0)==YES)
    {
      for (x=cx;x < num_lines;x++)
      {
        screen[x][0]=screen[x+1][0];
        memmove(screen[x]+1,screen[x+1]+1,strlen(screen[x+1]+1)+1);
      }

      Free_Line(num_lines);

      Puts(s(deld));
    }
  }
}


void pascal Bored_Insert(void)
{
  char temp[BUFLEN];

  int cx,x;

  WhiteN();
  
  OpusGets(temp,s(ins_bef));

  cx=atoi(temp);

  if (cx > 0 && cx <= num_lines && num_lines < max_lines-1)
  {
    if (Allocate_Line(num_lines+1))
      EdMemOvfl();

    for (x=num_lines-1;x >= cx;x--)
    {
      screen[x+1][0]=screen[x][0];
      memmove(screen[x+1]+1,screen[x]+1,strlen(screen[x]+1)+1);
    }

    screen[cx][0]='\r';
    screen[cx][1]='\0';

    if (! *linebuf)
      Puts(s(line_ins));
  }
  else Puts(toomanylines);
}



void pascal Bored_Edit(void)
{
  char temp[BUFLEN],
       search[BUFLEN],
       replace[BUFLEN],
       *p;

  int cx;

  WhiteN();

  OpusGets(temp,s(line_edit_num));

  cx=atoi(temp);

  if (cx > 0 && cx <= num_lines)
  {
    Putc('\n');
    Printf(s(blfmt1),cx,screen[cx]+1);
    
    WhiteN();

    if (usr.priv==NOVICE)
      Display_File(0,NULL,PRM(hlp_replace));

    OpusGetsL(search,BUFLEN,s(rep_what));

    if (*search && ! stristr(screen[cx]+1,search))
    {
      Printf(s(usrnotfnd),search);
      return;
    }

    if ((((signed int)LINELEN-(signed int)strlen(screen[cx])-5)+(signed int)strlen(search)) <= 0)
    {
      Puts(s(noroom));
      return;
    }

    Puts(s(editl1));

    if (! *search)
      Puts(s(editl2));
    else Printf(s(editl3),search);

    Printf(s(e_numch),
           (LINELEN-strlen(screen[cx])-5)+strlen(search));  
           /* Make sure to change formula above, too! */

    OpusGetsL(replace,min(BUFLEN,(LINELEN-strlen(screen[cx])-5)+strlen(search)),
              s(new_st));

    if (! *search)
    {
      memmove(screen[cx]+1+strlen(replace),screen[cx]+1,strlen(screen[cx]+1)+1);
      memmove(screen[cx]+1,replace,strlen(replace));

      Putc('\n');
      Printf(s(blfmt1),cx,screen[cx]+1);
      Putc('\n');
    }
    else
    {
      if ((p=stristr(screen[cx]+1,search)) != NULL) /* Substitute the replacement for the search. */
      {
        if (strlen(replace) >= strlen(search))
        {
          if (strlen(replace) != strlen(search))
            memmove(p+(strlen(replace)-strlen(search)),p,strlen(p)+1);

          memmove(p,replace,strlen(replace));
        }
        else
        {
          memmove(p,replace,strlen(replace));
          memmove(p+strlen(replace),p+(strlen(replace)+(strlen(search)-strlen(replace))),strlen(p+strlen(replace)));
        }
      }

      Putc('\n');
      Printf(s(blfmt1),cx,screen[cx]+1);
      Putc('\n');
    }
  }
}


void pascal Bored_Quote(void)
{
  struct _msg msg;

  char temp[PATHLEN],
       quotebuf[MAX_LINELEN],
       linetype[1],
       initials[10],
       last_msg_attr,
       nonstop,
       *p[1];

  int qfile,
      startline,
      endline,
      total_lines,
      display_msg,
      lc,
      x;

  if (!isareply)
  {
    Puts(s(not_reply));
    Press_ENTER();
    return;
  }

  for (display_msg=TRUE;display_msg;)
  {
    Putc('\n');
    OpusGets(temp,s(qstart));

    if (eqstri(temp,qmark))
    {
      lc=1;
      display_msg=TRUE;
      startline=1;
      endline=0x7fff;
    }
    else if (! *temp)
      return;
    else
    {
      display_msg=FALSE;
      startline=atoi(temp);

      OpusGets(temp,s(qend));

      if (! *temp)
        return;

      endline=atoi(temp);

      if (startline < 1)
        startline=1;

      if (endline < 1)
        endline=1;

      if (endline < startline)
        endline=startline;

      if (endline-startline >= 20)
        Puts(s(sstmt));
    }

    display_line=display_col=0;

    Putc('\n');

    sprintf(temp,d_msg,last_msg);

    if ((qfile=shopen(temp,O_RDONLY | O_BINARY))==-1)
      return;

    read(qfile,(char *)&msg,sizeof(struct _msg));

    Parse_Initials(msg.from,initials);

    nonstop=0;

    for (total_lines=0,last_msg_attr=0,x=1;total_lines < endline && x > 0;)
    {
      *p=quotebuf;
      x=Msg_Read_Lines(qfile,1,usrwidth-strlen(initials)-8,p,
                       linetype,&last_msg_attr);

      if (linetype[0] & MSGLINE_seenby && !GEPriv(usr.priv,prm.seenby_priv))
        continue;
      else if (linetype[0] & MSGLINE_kludge && !GEPriv(usr.priv,prm.ctla_priv))
        continue;
      else if (linetype[0] & MSGLINE_end)
        break;

      if (++total_lines >= startline && num_lines < max_lines-1)
      {
        if (! display_msg)
        {
          if (Allocate_Line(num_lines+1))
            EdMemOvfl();

          linenum=num_lines;

          screen[linenum][0]='\r';
          strcpy(screen[linenum]+1," ");
          strcat(screen[linenum]+1,initials);
          strcat(screen[linenum]+1,"> ");
          strcat(screen[linenum]+1,quotebuf);
        }

        Printf(GRAY "%2d: " CYAN " %s> %s\n",
               display_msg ? lc++ : linenum++,initials,quotebuf);

        if (display_msg && MoreYnBreak(&nonstop,CYAN))
          break;
      }
    }

    close(qfile);
  }
}



int pascal Local_Editor(struct _msg *msg,int msgfile)
{
  FILE *outmsgfile;
  int inmsgfile;

  char temp[PATHLEN],
       msgtemp[PATHLEN],
       outline[MAX_LINELEN],
       initials[10],
       linetype[1],
       last_msg_attr,
       *p[1];

  int aborted,
      num_n;

  last_msg_attr=0;

  sprintf(msgtemp,msgtemp_name,PRM(temppath),task_num);

  Parse_Initials(msg->to,initials);

  if (isareply || msgfile != -1)
  {
    if (msgfile==-1)    /* If it's a reply */
    {
      sprintf(temp,d_msg,last_msg);
      inmsgfile=shopen(temp,O_RDONLY | O_BINARY);
    }
    else inmsgfile=msgfile;  /* Else it's a change, so assume prior file */

    if (inmsgfile != -1)
    {
      /* Throw away message header */

      lseek(inmsgfile,(long)sizeof(struct _msg),SEEK_SET);

      if ((outmsgfile=shfopen(msgtemp,fopen_write,O_WRONLY | O_CREAT | O_TRUNC)) != NULL)
      {
        num_n=0;
        
        *p=outline;
        while (Msg_Read_Lines(inmsgfile,1,usr.width-strlen(initials)-6,
                              p,linetype,&last_msg_attr) > 0)
        {
          if (linetype[0] & MSGLINE_seenby)
          {
            if (! GEPriv(usr.priv,prm.seenby_priv))
              continue;
          }
          else if (linetype[0] & MSGLINE_kludge)
          {
            if (! GEPriv(usr.priv,prm.ctla_priv) || msgfile != -1)
              continue;
          }
    
          if (msgfile != -1 && strnicmp(outline,s(max_tear),16)==0)
            break;

          if (msgfile==-1)
            fprintf(outmsgfile," %s> %s\n",initials,outline);
          else
          {
            /* Strip any blank lines at the end of the file, so buffer      *
             * em, and only write when necessary.                           */
      
            if (! *outline)
              num_n++;
            else
            {
              if (num_n)
              {
                while (num_n--)
                  fputc('\n',outmsgfile);
                
                num_n=0;
              }
  
              fprintf(outmsgfile,"%s\n",outline);
            }
          }
        }

        fclose(outmsgfile);
      }

      if (msgfile==-1)    /* Only close if it's a reply */
        close(inmsgfile);
    }
  }

  Save_Directory(PRM(temppath));

  sprintf(temp,*PRM(local_editor)=='!' ? PRM(local_editor)+1 :
          PRM(local_editor),msgtemp);

  Outside(NULL,NULL,OUTSIDE_run | OUTSIDE_stay,temp,FALSE,CTL_none,
          RESTART_menu,NULL);

  Restore_Directories();
  Save_Directory(area.msgpath);

  if ((outmsgfile=shfopen(msgtemp,fopen_readb,O_RDONLY | O_BINARY))==NULL)
    return ABORT;

  aborted=Save_Msg(msg,outmsgfile,TRUE,msgfile==-1 ? 0 : lmsg,0);

  fclose(outmsgfile);

  if (aborted)
    return ABORT;
  else
  {
    unlink(msgtemp);
    return LOCAL_EDIT;
  }
}


